/*
 * Copyright 2014 Chris Banes
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.android.common.view;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.View;
import android.widget.LinearLayout;

class SlidingTabStrip extends LinearLayout {

	private static final int DEFAULT_INDICATOR_HEIGHT_DIPS = 3;

	private int mSelectedIndicatorHeight;
	private final Paint mSelectedIndicatorPaint;

	private int mSelectedPosition;
	private float mSelectionOffset;

	SlidingTabStrip(Context context) {
		this(context, null);
	}

	SlidingTabStrip(Context context, AttributeSet attrs) {
		super(context, attrs);
		setWillNotDraw(false);

		final float density = getResources().getDisplayMetrics().density;
		mSelectedIndicatorHeight = (int) (DEFAULT_INDICATOR_HEIGHT_DIPS * density);
		mSelectedIndicatorPaint = new Paint();
	}

	void setSelectedIndicatorColor(int color) {
		mSelectedIndicatorPaint.setColor(color);
		invalidate();
	}

	void setSelectedIndicatorHeight(int height) {
		mSelectedIndicatorHeight = height;
		invalidate();
	}

	void onViewPagerPageChanged(int position, float positionOffset) {
		mSelectedPosition = position;
		mSelectionOffset = positionOffset;
		invalidate();
	}

	@Override
	protected void onDraw(Canvas canvas) {
		final int height = getHeight();
		final int childCount = getChildCount();

		// Thick colored underline below the current selection
		if (childCount > 0) {
			View selectedTitle = getChildAt(mSelectedPosition);
			int left = selectedTitle.getLeft();
			int right = selectedTitle.getRight();

			if (mSelectionOffset > 0f && mSelectedPosition < (getChildCount() - 1)) {
				// Draw the selection partway between the tabs
				View nextTitle = getChildAt(mSelectedPosition + 1);
				left = (int) (mSelectionOffset * nextTitle.getLeft() +
						(1.0f - mSelectionOffset) * left);
				right = (int) (mSelectionOffset * nextTitle.getRight() +
						(1.0f - mSelectionOffset) * right);
			}

			canvas.drawRect(left, height - mSelectedIndicatorHeight, right,
					height, mSelectedIndicatorPaint);
		}
	}

}